use ctap2_proto::{
    prelude::{
        client_pin::{self, auth_protocol::platform::Session, Permission, PinUvAuthToken},
        credential, management,
    },
    Ctap2_2Authenticator,
};
use hmac::{Hmac, Mac};
use sha2::Digest;

use super::{get_session, AUTHENTICATOR};

#[test]
fn test_get_credentials_metadata() {
    let mut guard = AUTHENTICATOR.lock().unwrap();
    let mut authenticator = guard.as_mut().unwrap();

    let session = get_session(&mut authenticator);

    let pin = "87654321";

    // pinHashEnc: The result of calling encrypt(shared secret,
    // LEFT(SHA-256(curPin), 16)).
    let pin_hash_encrypted = session
        .encrypt(&[*sha2::Sha256::digest(pin.as_bytes())
            .split_array_ref::<16>()
            .0])
        .unwrap()[0];

    let req = client_pin::Request::GetPinToken {
        version: client_pin::auth_protocol::Version::One,
        key_agreement: session.platform_key_agreement_key().clone(),
        pin_hash_encrypted,
    };

    let res = authenticator.client_pin(req).unwrap();

    let client_pin::Response::GetPinToken { pin_uv_auth_token } = res else {
        panic!("expected GetPinToken response");
    };

    let PinUvAuthToken::Long(pin_uv_auth_token) = pin_uv_auth_token else {
        panic!("Expected long pinuvauthtoken");
    };

    // Decrypt pin_uv_auth_token
    let encrypted: &[_; 2] = unsafe { std::mem::transmute(&pin_uv_auth_token) };
    let pin_uv_auth_token: [u8; 32] = unsafe { std::mem::transmute(session.decrypt(encrypted)) };

    // pinAuth (0x04): LEFT(HMAC-SHA-256(pinToken, getCredsMetadata (0x01)), 16).
    let mut mac = Hmac::<sha2::Sha256>::new_from_slice(pin_uv_auth_token.as_ref()).unwrap();

    mac.update(&[0x01]);

    let pin_uv_auth_param: [u8; 16] = mac
        .finalize()
        .into_bytes()
        .split_array_ref::<16>()
        .0
        .to_owned();

    let req = management::Request::GetCredentialsMetadata {
        pin_uv_auth_protocol: client_pin::auth_protocol::Version::One,
        pin_uv_auth_param,
    };
    let res = authenticator.credential_management(req).unwrap();
}
